/* -----------------------------------------------------------------------------
    BLP Plugin for Imagine

-------------------------------------------------------------------------- */
#include "Main.H"
#include <cstring>
#include "blpjpeg.h"
#include "blp.h"
#include "dxt.h"
#include "MCCQ.h"


#define FILETYPE_BLP "Blizzard picture format"
#define EXTENSION_BLP "BLP\0"
#define DXTCOMPRESSION "DXT compression"
#define BLP0 0x30504c42
#define BLP1 0x31504c42
#define BLP2 0x32504c42
#define BLP1ALPHA 8
#define PALETTED 1
#define DXTCOMPRESSED 2
#define UNCOMPRESSED 3
#define DXT1 0
#define DXT3 0x10000
#define DXT5 0x70000


//#pragma pack(1)

typedef struct BLPHeader {
    unsigned long Signature;
    unsigned long PixelCompression;
    unsigned long Flags;
    unsigned long Width;
    unsigned long Height;
    union {
        struct {
            unsigned long alphaEncoding;
            unsigned long flags2;
            unsigned long offsets[16];
            unsigned long lengths[16];
        }blp1;
        struct {
            unsigned long offsets[16];
            unsigned long lengths[16];
        }blp2;
    }HeaderEnd;
} BLPHeader, *lpBLPHeader;
//#pragma pack()
typedef unsigned long long int QWORD;

static IMAGINELOADPARAM* gloadparam;
static IMAGINECALLBACKPARAM callbackprm;
BOOL callbackjpg(int pg)
{
    callbackprm.current = pg;
    return gloadparam->callback.proc(&callbackprm);
}

static BOOL IMAGINEAPI checkFile(IMAGINEPLUGINFILEINFOTABLE *fileInfoTable, IMAGINELOADPARAM *loadParam, int flags)
{
    if (loadParam->length>160) {
        lpBLPHeader header = (lpBLPHeader)loadParam->buffer;
        if ((header->Signature == BLP0 || header->Signature == BLP1 || header->Signature == BLP2) && header->Height<32769 && header->Height&&header->Width&&header->Width<32769 && header->PixelCompression<2)
            return TRUE;
    }
    return FALSE;
}


static LPIMAGINEBITMAP IMAGINEAPI loadFile(IMAGINEPLUGINFILEINFOTABLE *fileInfoTable, IMAGINELOADPARAM *loadParam, int flags)
{
    LPIMAGINEBITMAP bitmap = NULL;
    if (fileInfoTable->iface != NULL) {
        const IMAGINEPLUGININTERFACEVTBL *lpVtbl = fileInfoTable->iface->lpVtbl;
        if (loadParam->length>160) {
            lpBLPHeader header = (lpBLPHeader)loadParam->buffer;
            DWORD firstMipmap, firstLenght, ImgHeaderof, *sourced, *destd;
            if (header->Signature == BLP2) {
                firstMipmap = header->HeaderEnd.blp2.offsets[0];
                firstLenght = header->HeaderEnd.blp2.lengths[0];
                ImgHeaderof = 0x94U;
            }
            else {
                firstMipmap = header->HeaderEnd.blp1.offsets[0];
                firstLenght = header->HeaderEnd.blp1.lengths[0];
                ImgHeaderof = 0x9cU;
            }
            if (firstMipmap + firstLenght > loadParam->length)firstLenght = loadParam->length - firstMipmap;
            if ((header->Signature == BLP0 || header->Signature == BLP1 || header->Signature == BLP2) && firstLenght&&header->Height<32769 && header->Height&&header->Width&&header->Width<32769 && header->PixelCompression<2 && loadParam->length > firstMipmap) {
                if (!(flags&IMAGINELOADPARAM_GETINFO)) {
                    LONG bitCount = 32;
                    BYTE* source, *dest;
                    DWORD stp, ofs;
                    DWORD buf[256];
                    if (header->PixelCompression) {
                        if (ImgHeaderof == 0x9cU) { if (!header->Flags || firstLenght<header->Width*header->Height)bitCount = 8; }
                        else if ((header->Flags & 0xff) == PALETTED && (!(header->Flags & 0xff00) || firstLenght<header->Width*header->Height))bitCount = 8;
                    }
                    bitmap = lpVtbl->Create(header->Width, header->Height, bitCount, flags);
                    if (bitmap != NULL) {
                        if (header->PixelCompression) {
                            if (ImgHeaderof == 0x9CU || (header->Flags & 0xff) == PALETTED) {
                                
                                lpVtbl->SetCompression(bitmap, IMAGINECOMPRESSION_NONE);
                                if (firstMipmap>ImgHeaderof) {
                                    //Paletted
                                    if (bitCount == 8) {
                                        DWORD count = (loadParam->length - ImgHeaderof) >> 2;
                                        if (count>256)count = 256;
                                        lpVtbl->UtilImportPalette((LPPALETTEENTRY)buf, (BYTE*)loadParam->buffer + ImgHeaderof, count, 2);
                                        lpVtbl->SetPalette(bitmap, (LPPALETTEENTRY)buf);
                                        source = (BYTE*)(loadParam->buffer) + firstMipmap;
                                        callbackprm = { 0 };
                                        stp = header->Width;
                                        callbackprm.dib = bitmap;
                                        callbackprm.param = loadParam->callback.param;
                                        callbackprm.overall = header->Height;
                                        for (callbackprm.current=0; (DWORD)callbackprm.current<header->Height; callbackprm.current++) {
                                            if ((callbackprm.current+1)*stp>firstLenght) {
                                                if (callbackprm.current*stp < firstLenght)
                                                    memcpy((BYTE*)(lpVtbl->GetLineBits(bitmap, callbackprm.current)), source + callbackprm.current*stp, firstLenght - callbackprm.current*stp);
                                            }
                                            else memcpy((BYTE*)(lpVtbl->GetLineBits(bitmap, callbackprm.current)), source + callbackprm.current*stp, stp);
                                            if ((flags&IMAGINELOADPARAM_CALLBACK) && (!loadParam->callback.proc(&callbackprm))) {
                                                loadParam->errorCode = IMAGINEERROR_ABORTED;
                                                break;
                                            }
                                        }
                                        //alpha
                                        if (!header->Flags || (header->Flags & 0xffff) == PALETTED) {
                                            lpVtbl->SetTransMethod(bitmap, IMAGINETRANSMETHOD_NONE);
                                        }
                                        else if (ImgHeaderof == 0x9cU&&header->Flags) {
                                            lpVtbl->SetTransMethod(bitmap, IMAGINETRANSMETHOD_ALPHABLEND);
                                        }
                                    }

                                    //Paletted with alpha
                                    else {
                                        source = (BYTE*)(loadParam->buffer) + firstMipmap;
                                        sourced = (DWORD*)((BYTE*)loadParam->buffer + ImgHeaderof);
                                        callbackprm = { 0 };
                                        stp = header->Width;
                                        callbackprm.dib = bitmap;
                                        callbackprm.param = loadParam->callback.param;
                                        callbackprm.overall = header->Height;
                                        DWORD alphaDepth, s, a;
                                        BYTE bof = 1, b;
                                        lpVtbl->SetTransMethod(bitmap, IMAGINETRANSMETHOD_ALPHABLEND);
                                        if (ImgHeaderof == 0x9cU)alphaDepth = 8;
                                        else alphaDepth = (header->Flags & 0xf00) >> 8;
                                        ofs = header->Height*header->Width;
                                        for (callbackprm.current=0; (DWORD)callbackprm.current<header->Height; callbackprm.current++) {
                                            destd = (DWORD*)(lpVtbl->GetLineBits(bitmap, callbackprm.current));
                                            if (alphaDepth == 8) {
                                                for (DWORD x = 0; x < header->Width; x++, source++) {
                                                    s = *(sourced + (*source));
                                                    //alpha
                                                    if (ofs <= firstLenght) {
                                                        a = *(source + ofs);
                                                        s = (a << 24)|(s&0xffffff);
                                                    }
                                                    *destd++ = s;
                                                }
                                            }
                                            else if (alphaDepth == 4) {
                                                for (DWORD x = 0; x < header->Width; x++, source++) {
                                                    s = *(sourced + (*source));
                                                    //alpha
                                                    if (ofs <= firstLenght) {
                                                        b = *(source + ofs);
                                                        if (!bof) {
                                                            b = b & 0xf0; b |= b >> 4;
                                                        }
                                                        else {
                                                            b = b & 0xf; b |= b << 4;
                                                            ofs--;
                                                        }
                                                        a = b;
                                                        s = (a << 24) | (s & 0xffffff);
                                                        bof = !bof;
                                                    }
                                                    *destd++ = s;
                                                }
                                            }
                                            else if (alphaDepth == 2) {
                                                for (DWORD x = 0; x < header->Width; x++, source++) {
                                                    s = *(sourced + (*source));
                                                    //alpha
                                                    if (ofs <= firstLenght) {
                                                        s &= 0xffffff;
                                                        b = *(source + ofs);
                                                        if (b&bof)s |= 0x55000000;
                                                        bof += bof;
                                                        if (a&bof)s |= 0xaa000000;
                                                        bof += bof;
                                                        if (bof)ofs--;
                                                        else bof = 1;
                                                    }
                                                    *destd++ = s;
                                                }
                                            }
                                            else if (alphaDepth == 1) {
                                                for (DWORD x = 0; x < header->Width; x++, source++) {
                                                    s = *(sourced + (*source));
                                                    //alpha
                                                    if (ofs <= firstLenght) {
                                                        b = *(source + ofs);
                                                        if (b&bof)s |= 0xff000000;
                                                        else s &= 0xffffff;
                                                        bof += bof;
                                                        if (bof)ofs--;
                                                        else bof = 1;
                                                    }
                                                    *destd++ = s;
                                                }
                                            }
                                            if ((flags&IMAGINELOADPARAM_CALLBACK) && (!loadParam->callback.proc(&callbackprm))) {
                                                loadParam->errorCode = IMAGINEERROR_ABORTED;
                                                break;
                                            }

                                        }
                                    }
                                }
                                else loadParam->errorCode = IMAGINEERROR_READERROR;
                            }
                            //dxt
                            else if ((header->Flags & 0xff) == DXTCOMPRESSED) {
                                source = (BYTE*)(loadParam->buffer) + firstMipmap;
                                ofs = 0;
                                lpVtbl->SetTransMethod(bitmap, IMAGINETRANSMETHOD_ALPHABLEND);
                                stp = (header->Flags & 0xff0000) ? 16 : 8;
                                callbackprm = { 0 };
                                callbackprm.dib = bitmap;
                                callbackprm.param = loadParam->callback.param;
                                callbackprm.current = 0;
                                callbackprm.overall = header->Height;
                                int dn=1;
                                if ((header->Flags & 0xff0000) == DXT1) dn = 1;
                                else if ((header->Flags & 0xff0000) == DXT3) dn = 3;
                                else if ((header->Flags & 0xff0000) == DXT5) dn = 5;
                                else loadParam->errorCode = IMAGINEERROR_UNSUPPORTEDTYPE;
                                for (callbackprm.current = 0; (DWORD)callbackprm.current <header->Height; callbackprm.current += 4) {
                                    DWORD* lines[4];
                                    lines[0]= (DWORD*)(lpVtbl->GetLineBits(bitmap, callbackprm.current));
                                    lines[1] = (DWORD*)(lpVtbl->GetLineBits(bitmap, callbackprm.current+1));
                                    lines[2] = (DWORD*)(lpVtbl->GetLineBits(bitmap, callbackprm.current+2));
                                    lines[3] = (DWORD*)(lpVtbl->GetLineBits(bitmap, callbackprm.current+3));
                                    for (DWORD x = 0; x<header->Width && ((x & 3) || ofs + stp <= firstLenght); x++) {
                                        if (!(x & 3)) {
                                            dxt::DecompressDxt((BYTE*)buf, source + ofs,dn);
                                            ofs += stp;
                                        }
                                        for (int i = 0; i<4 && (DWORD)(callbackprm.current + i) <header->Height; i++) {
                                            DWORD s = buf[(x & 3) + (i << 2)];
                                            
                                            lines[i][x] = (((s << 16) | (s >> 16)) & 0xff00ff) | (s & 0xff00ff00);
                                        }
                                    }
                                    if ((flags&IMAGINELOADPARAM_CALLBACK) && (!loadParam->callback.proc(&callbackprm))) {
                                        loadParam->errorCode = IMAGINEERROR_ABORTED;
                                        break;
                                    }
                                }
                                if (lpVtbl->IsUnicode)lpVtbl->SetCompressionText(bitmap, (LPCTSTR)UNICODE_TEXT(DXTCOMPRESSION));
                                else lpVtbl->SetCompressionText(bitmap, DXTCOMPRESSION);
                            }
                            //UNCOMPRESSED
                            else if ((header->Flags & 0xff) == UNCOMPRESSED){
                                lpVtbl->SetCompression(bitmap, IMAGINECOMPRESSION_NONE);
                                lpVtbl->SetTransMethod(bitmap, IMAGINETRANSMETHOD_ALPHABLEND);
                                source = (BYTE*)(loadParam->buffer) + firstMipmap;
                                callbackprm = { 0 };
                                callbackprm.dib = bitmap;
                                stp = header->Width*4;
                                callbackprm.param = loadParam->callback.param;
                                callbackprm.overall = header->Height;
                                for (callbackprm.current = 0; (DWORD)callbackprm.current<header->Height; callbackprm.current++) {
                                    if ((callbackprm.current + 1)*stp>firstLenght) {
                                        if (callbackprm.current*stp < firstLenght)
                                            memcpy((BYTE*)(lpVtbl->GetLineBits(bitmap, callbackprm.current)), source + callbackprm.current*stp, firstLenght - callbackprm.current*stp);
                                    }
                                    else memcpy((BYTE*)(lpVtbl->GetLineBits(bitmap, callbackprm.current)), source + callbackprm.current*stp, stp);
                                    if ((flags&IMAGINELOADPARAM_CALLBACK) && (!loadParam->callback.proc(&callbackprm))) {
                                        loadParam->errorCode = IMAGINEERROR_ABORTED;
                                        break;
                                    }
                                }
                            }

                            else loadParam->errorCode = IMAGINEERROR_UNSUPPORTEDTYPE;
                        }
                        //JPEG
                        else {
                            gloadparam = loadParam;
                            lpVtbl->SetCompression(bitmap, IMAGINECOMPRESSION_JPEG);
                            lpVtbl->SetTransMethod(bitmap, IMAGINETRANSMETHOD_ALPHABLEND);
                            source = (BYTE*)loadParam->buffer+ ImgHeaderof;
                            DWORD length = *(DWORD*)source;
                            source += 4;
                            DWORD a = loadParam->length;

                            if (length + ImgHeaderof + 4 > a)length = a - 4 - ImgHeaderof;
                            if (!length) {
                                source = (BYTE*)loadParam->buffer + firstMipmap;
                            }
                            else if(ImgHeaderof+length+4!= firstMipmap){
                                dest = (BYTE*)lpVtbl->Alloc(length + firstLenght);
                                if (!dest) {
                                    loadParam->errorCode = IMAGINEERROR_OUTOFMEMORY;
                                    return bitmap;
                                }
                                memcpy(dest, source, length);
                                memcpy(dest + length, (BYTE*)loadParam->buffer + firstMipmap, firstLenght);
                                source = dest;
                            }
                            length += firstLenght;
                            blpjpeg::jpegDecompress* pjpegDecompress = (blpjpeg::jpegDecompress*)lpVtbl->Alloc(sizeof(blpjpeg::jpegDecompress)+ ((header->Height+7)>>3)*((header->Width+7)>>3)*512);
                            if(!pjpegDecompress){ 
                                loadParam->errorCode = IMAGINEERROR_OUTOFMEMORY;
                                return bitmap;
                            }
                            pjpegDecompress->bitmap = (BYTE*)(lpVtbl->GetBits(bitmap));
                            pjpegDecompress->length = length;
                            pjpegDecompress->source = source;
                            pjpegDecompress->height = header->Height;
                            pjpegDecompress->width = header->Width;
                            pjpegDecompress->dehuff = (short*)(pjpegDecompress + 1);
                            pjpegDecompress->callback = NULL;
                            if (flags&IMAGINELOADPARAM_CALLBACK) {
                                callbackprm = { 0 };
                                callbackprm.dib = bitmap;
                                callbackprm.param = loadParam->callback.param;
                                callbackprm.current = 0;
                                callbackprm.overall = 100;
                                pjpegDecompress->callback = &callbackjpg;
                            }
                            blpjpeg::blpjpegDecompress(pjpegDecompress);
                            
                            
                           
                        }
                    }
                    else loadParam->errorCode = IMAGINEERROR_OUTOFMEMORY;
                }
                else loadParam->errorCode = IMAGINEERROR_GETINFONOTSUPPORTED;
            }
            else loadParam->errorCode = IMAGINEERROR_INVALIDDATA;
        }
        else loadParam->errorCode = IMAGINEERROR_READERROR;
    }
    return bitmap;
}


extern DWORD dl3quantfast(unsigned char *inbuf, unsigned long pixels, unsigned char* userpal, DWORD colorDepth);
extern DWORD dl3quantslow(unsigned char *inbuf, unsigned long pixels, unsigned char* userpal, DWORD colorDepth);
extern void neuquantcq(unsigned char *thepic, unsigned long pixels, unsigned char* pal, unsigned long colorDepth,DWORD cycles);

static BOOL IMAGINEAPI saveFile(IMAGINEPLUGINFILEINFOTABLE *fileInfoTable, LPIMAGINEBITMAP bitmap, IMAGINESAVEPARAM *saveParam, int flags)
{
    BOOL result = FALSE;
    if (fileInfoTable->iface != NULL) {
        const IMAGINEPLUGININTERFACEVTBL *lpVtbl = fileInfoTable->iface->lpVtbl;
        LONG colorDepth = lpVtbl->GetBitCount(bitmap);
        DWORD a = lpVtbl->GetTransMethod(bitmap), a1;
        if ((colorDepth == 8 || colorDepth == 24 || colorDepth == 32) && (a == IMAGINETRANSMETHOD_NONE || a == IMAGINETRANSMETHOD_ALPHABLEND)) {
            if (flags&IMAGINESAVEPARAM_TEST)result = TRUE;
            else {
                lpBLPHeader header;
                DWORD width, height, *sourced, s;
                BYTE *dest;
                height = lpVtbl->GetHeight(bitmap);
                width = lpVtbl->GetWidth(bitmap);
                colorDepth = colorDepth >> 3;
                LONG length = 4*width*height;
                s = length + 0x400;

                
                length += length + 0x1800;
                BYTE* buf = (BYTE*)lpVtbl->Alloc(length + s);
                saveParam->sb = lpVtbl->sbAlloc(length, 32768);
                header = (lpBLPHeader)(buf + s);
                dest = buf + s;
                if (saveParam->sb != NULL&&buf != NULL) {
                    result = TRUE;
                    BYTE* source;
                    DWORD saveopt = *(DWORD*)(saveParam->option.reserved + 4);
                    DWORD *destd;
                    header->Height = height;
                    header->Width = width;
                    if (saveopt < 3)header->Signature = BLP1;
                    else header->Signature = BLP2;
                    if (saveopt == 0 || saveopt == 1|| saveopt == 3 || saveopt == 4)header->PixelCompression = 0; else header->PixelCompression = 1;
                    DWORD alphaDepth = 0;
                    if (a == IMAGINETRANSMETHOD_ALPHABLEND&&colorDepth != 3) {

                        alphaDepth = 8;
                        if(saveopt == 7)alphaDepth = 4;
                        else if (saveopt == 8)alphaDepth = 1;
                    }

                    a = 0;
                    if (saveopt < 3) {
                        dest += 0x9c;
                        header->HeaderEnd.blp1.flags2 = 1;
                        if (alphaDepth) {
                            a = 8;
                            header->HeaderEnd.blp1.alphaEncoding = 4;
                        }
                        else header->HeaderEnd.blp1.alphaEncoding = 5;
                    }
                    else {
                        dest += 0x94;
                        a = 0x1000000 + (alphaDepth << 8);
                        if (saveopt == 5|| saveopt == 7|| saveopt == 8)a |= 0x80001;//blp2 paletted
                        else if (saveopt == 6) {//dxt
                            if (alphaDepth > 1)a |= 0x70002;
                            else a |= 0x2;
                        }
                    }
                    header->Flags = a;

                    //copy bitmap
                    destd = (DWORD*)(buf + 0x400);
                    if (saveopt == 6) {
                        //dxt
                        if (colorDepth == 1) {
                            lpVtbl->GetPalette(bitmap, (LPPALETTEENTRY)buf);

                            DWORD* pal = (DWORD*)buf;
                            if (alphaDepth)
                                for (DWORD y = 0; y < height; y++) {
                                    source = (BYTE*)lpVtbl->GetLineBits(bitmap, y);
                                    for (DWORD x = 0; x < width; x++) {
                                        *destd++ = pal[*source++];
                                    }

                                }
                            else
                                for (DWORD y = 0; y < height; y++) {
                                    source = (BYTE*)lpVtbl->GetLineBits(bitmap, y);
                                    for (DWORD x = 0; x < width; x++) {
                                        *destd++ = pal[*source++] | 0xff000000;
                                    }

                                }
                        }
                        else if (colorDepth == 3) {
                            for (DWORD y = 0; y < height; y++) {
                                source = (BYTE*)lpVtbl->GetLineBits(bitmap, y);
                                for (DWORD x = 0; x < width; x++) {
                                    s = *source++ | 0xff00;
                                    s = s << 8;
                                    s += *source++;
                                    s = s << 8;
                                    s += *source++;
                                    *destd++ = s;
                                }
                            }
                        }
                        else {
                            if (alphaDepth)
                                for (DWORD y = 0; y < height; y++) {
                                    sourced = (DWORD*)lpVtbl->GetLineBits(bitmap, y);
                                    for (DWORD x = 0; x < width; x++) {
                                        s = *sourced++;
                                        *destd++ = (((s << 16) | (s >> 16)) & 0xff00ff) | (s & 0xff00ff00);
                                    }
                                }
                            else {
                                BYTE* dest1 = (BYTE*)destd;
                                for (DWORD y = 0; y < height; y++) {
                                    source = (BYTE*)lpVtbl->GetLineBits(bitmap, y);
                                    for (DWORD x = 0; x < width; x++) {
                                        *dest1++ = *(source + 2);
                                        *dest1++ = *(source + 1);
                                        *dest1++ = *source;
                                        *dest1++ = 0xff;
                                        source += 4;
                                    }
                                }
                            }
                        }
                    }

                    else {
                        
                        if (colorDepth == 1) {
                            PALETTEENTRY tmpPal[256];
                            lpVtbl->GetPalette(bitmap, tmpPal);
                            lpVtbl->UtilExportPalette(tmpPal, buf, 256, 2);
                            /*for (count = 0; count<256; count++) {
                            s = *(sourced + count);
                            destd[count] = (((s << 16) | (s >> 16)) & 0xff00ff) | (s & 0xff00ff00);//bgra to rgba
                            }*/
                            DWORD* pal = (DWORD*)buf;
                            if (alphaDepth)
                                for (DWORD y = 0; y < height; y++) {
                                    source = (BYTE*)lpVtbl->GetLineBits(bitmap, y);
                                    for (DWORD x = 0; x < width; x++) {
                                        *destd++ = pal[*source++];
                                    }

                                }
                            else
                                for (DWORD y = 0; y < height; y++) {
                                    source = (BYTE*)lpVtbl->GetLineBits(bitmap, y);
                                    for (DWORD x = 0; x < width; x++) {
                                        *destd++ = pal[*source++] | 0xff000000;
                                    }

                                }
                        }
                        else if (colorDepth == 3) {
                            for (DWORD y = 0; y < height; y++) {
                                source = (BYTE*)lpVtbl->GetLineBits(bitmap, y);
                                for (DWORD x = 0; x < width; x++) {
                                    s = *(source+2) | 0xff00;
                                    s = s << 16;
                                    s += *(WORD*)source; source+=3;
                                    *destd++ = s;
                                }
                            }
                        }
                        else {
                            if (alphaDepth)
                                for (DWORD y = 0; y < height; y++) {
                                    memcpy(destd, lpVtbl->GetLineBits(bitmap, y), width * 4);
                                    destd+=width;
                                }
                            else {
                                for (DWORD y = 0; y < height; y++) {
                                    sourced = (DWORD*)lpVtbl->GetLineBits(bitmap, y);
                                    for (DWORD x = 0; x < width; x++) {
                                        *destd++ = *sourced++ + 0xff000000;
                                    }
                                }
                            }
                        }
                    }
                    

                    
                    blpjpeg::jpegCompress* pjpegCompress;
                    
                    if (saveopt == 2 || saveopt == 5 || saveopt == 7 || saveopt == 8) {
                        if (colorDepth != 1) {
                            a = *(DWORD*)(saveParam->option.reserved + 8);
                            if (a == 2) { 
                                a=dl3quantfast(buf + 0x400, height*width, buf, 4);
                                if(a<256)GetPaletteMCCQ(buf, (piped*)dest, buf + 0x400, height*width, 4);
                            }
                            else if (a == 3) {
                                a=dl3quantslow(buf + 0x400, height*width, buf, 4);
                                if (a<256)GetPaletteMCCQ(buf, (piped*)dest, buf + 0x400, height*width, 4);
                            }
                            else if(!a&&(height*width)>503)neuquantcq(buf + 0x400, height*width, buf, 4,100);
                            else if (a==1 && (height*width)>503)neuquantcq(buf + 0x400, height*width, buf, 4, 128);
                            else GetPaletteMCCQ(buf,(piped*)dest, buf+0x400, height*width, 4);
                        }
                        memcpy(dest, buf, 1024);
                        dest += 0x400;
                    }
                    DWORD* offs, *lens;
                    if (saveopt < 3) {
                        offs = header->HeaderEnd.blp1.offsets;
                        lens = header->HeaderEnd.blp1.lengths;
                    }
                    else {
                        offs = header->HeaderEnd.blp2.offsets;
                        lens = header->HeaderEnd.blp2.lengths;
                    }
                    if (saveopt == 0 || saveopt == 1|| saveopt == 3 || saveopt == 4) {
                        //JPEG
                        if (saveopt < 3)offs[0] = 0xa0;
                        else offs[0] = 0x98;
                        *(DWORD*)dest = 0;
                        dest += 4;
                        pjpegCompress= (blpjpeg::jpegCompress*)lpVtbl->Alloc(sizeof(blpjpeg::jpegCompress)+ ((header->Height + 7) >> 3)*((header->Width + 7) >> 3) * 512);
                        pjpegCompress->quality = *(DWORD*)saveParam->option.reserved;
                    }
                    else {
                        if (saveopt == 2)offs[0] = 0x49c;
                        else if (saveopt == 5 || saveopt == 7 || saveopt == 8)offs[0] = 0x494;
                        else offs[0] = 0x94;
                    }
                    DWORD currentI;
                    DWORD maxI = *(DWORD*)(saveParam->option.reserved + 12);
                    //convert
                    for (currentI = 0; currentI<maxI;) {
                        //JPEG
                        if (saveopt == 0 || saveopt == 1|| saveopt == 3 || saveopt == 4) {
                            pjpegCompress->height = height;
                            pjpegCompress->width = width;
                            pjpegCompress->inbuff = buf + 0x400;
                            pjpegCompress->workbuff = (short*)(pjpegCompress + 1);
                            pjpegCompress->outbuff = dest;
                            if((height*width)<4096||saveopt == 0|| saveopt == 3)
                                blpjpeg::blpjpegCompressBase(pjpegCompress);
                            else
                                blpjpeg::blpjpegCompressProgressive(pjpegCompress);
                            //length
                            length = pjpegCompress->outlenght;
                            dest += length;
                        }
                        //paletted
                        else if ((saveopt == 2 || saveopt == 5 || saveopt == 7 || saveopt == 8)) {
                            
                            sourced = (DWORD*)(buf + 0x400);
                            length = height*width;
                            BYTE* desta = dest + length;
                            BYTE bof = 1;
                            DWORD min, minj;
                            long var;
                            BYTE b;

                            if (alphaDepth == 1)*desta = 0;
                            for (DWORD i = length; i--; sourced++, dest++) {
                                a = *sourced;

                                source = buf;
                                min = 0xffff;
                                for (DWORD j = 0; j<256; j++) {
                                    b = (BYTE)a;
                                    var = b - *source++;
                                    s=var * var;
                                    b = (BYTE)(a >> 8);
                                    var = b - *source++;
                                    s += var * var;
                                    b = (BYTE)(a>>16);
                                    var = b - *source++;
                                    s += var * var;
                                    source++;
                                    if (s<min) { min = s; minj = j; }
                                }

                                *dest = (BYTE)minj;
                                //alpha
                                if (alphaDepth) {
                                    if (alphaDepth == 8)*desta++ = (BYTE)(a >> 24);
                                    else if (alphaDepth == 4) {
                                        if (bof) {
                                            BYTE b = (BYTE)(a >> 28);
                                            b &= 0xf;
                                            *desta = b;
                                        }
                                        else {
                                            BYTE b = (BYTE)(a >> 24);
                                            b &= 0xf0;
                                            *desta |= b;
                                            desta++;
                                        }
                                        bof = !bof;
                                    }
                                    else if (alphaDepth == 1) {
                                        if ((long)a<0)*desta |= bof;
                                        bof += bof;
                                        if (!bof) {
                                            bof++; desta++; *desta = 0;
                                        }
                                    }
                                }
                            }
                            sourced = (DWORD*)(buf + 0x400);
                            if (alphaDepth == 1)a = (length + 7) >> 3;
                            else if (alphaDepth == 4)a = (length + 1) >> 1;
                            else if (alphaDepth == 8)a = length;
                            else a = 0;
                            dest += a;
                            //length
                            length += a;
                        }
                        //dxt
                        else {
                            dxt::CompressDxt((DWORD*)(buf + 0x400),dest, height,width, alphaDepth);
                            //length
                            if (alphaDepth)length = ((height + 3) & 0xfffffffc)*((width + 3) & 0xfffffffc);
                            else length = ((height + 3) >> 2)*((width + 3) >> 2) * 8;
                            dest += length;
                        }

                        lens[currentI] = length;
                        if ((width + height)>2) {
                            length += offs[currentI];
                            offs[++currentI] = length;
                            if (currentI == maxI) {
                                currentI--;
                                break;
                            }
                        }
                        else break;
                        //resize
                        sourced = destd = (DWORD*)(buf + 0x400);
                        if (width == 1) {
                            if (height != 1) {
                                height = height >> 1;
                                for (DWORD y = 0; y<height; y++) {
                                    s = *sourced++;
                                    a = (s & 0xff00ff00) >> 8;
                                    a1 = s & 0xff00ff;
                                    s = *sourced++;
                                    a += (s & 0xff00ff00) >> 8;
                                    a1 += s & 0xff00ff;
                                    destd[y] = ((a << 7) & 0xff00ff00) + ((a1 >> 1) & 0xff00ff);
                                }
                            }
                        }
                        else if (height == 1) {
                            if (width != 1) {
                                width = width >> 1;
                                for (DWORD x = 0; x<width; x++) {
                                    s = *sourced++;
                                    a = (s & 0xff00ff00) >> 8;
                                    a1 = s & 0xff00ff;
                                    s = *sourced++;
                                    a += (s & 0xff00ff00) >> 8;
                                    a1 += s & 0xff00ff;
                                    destd[x] = ((a << 7) & 0xff00ff00) + ((a1 >> 1) & 0xff00ff);
                                }
                            }
                        }
                        else {
                            height = height >> 1;
                            for (DWORD y = 0; y<height; y++) {
                                for (DWORD x = 0; x<(width >> 1); x++) {
                                    s = *sourced;
                                    a = (s & 0xff00ff00) >> 8;
                                    a1 = s & 0xff00ff;
                                    s = *(sourced + width);
                                    a += (s & 0xff00ff00) >> 8;
                                    a1 += s & 0xff00ff;
                                    sourced++;
                                    s = *sourced;
                                    a += (s & 0xff00ff00) >> 8;
                                    a1 += s & 0xff00ff;
                                    s = *(sourced + width);
                                    a += ((s & 0xff00ff00) >> 8) + 1;
                                    a1 += (s & 0xff00ff) + 1;
                                    *destd++ = ((a << 6) & 0xff00ff00) + ((a1 >> 2) & 0xff00ff);
                                    sourced++;
                                }
                                if (width & 1)sourced++;
                                sourced += width;
                            }
                            width = width >> 1;
                        }
                    }
                    //compress the jpg headers
                    if ((saveopt == 0 || saveopt == 1|| saveopt == 3 || saveopt == 4)&& currentI>1) {
                        dest = (BYTE*)header;
                        DWORD bn;
                        for (bn = 0; bn < lens[currentI]; bn++) {
                            BYTE cb = *(dest + offs[0] + bn);
                            DWORD i;
                            for (i = currentI; i; i--) {
                                if (cb != *(dest + offs[i] + bn))break;
                            }
                            if (i)break;
                        }
                        if (saveopt < 3)*(DWORD*)(dest + 0x9c) = bn;
                        else *(DWORD*)(dest + 0x94) = bn;
                        lens[0] -= bn;
                        offs[0] += bn;
                        for (DWORD i = 1; i <= currentI; i++) {
                            lens[i] -= bn;
                            memmove(dest + offs[i] - bn*(i - 1), dest + offs[i] + bn, lens[i]);
                            offs[i] -= bn*(i - 1);
                        }
                    }

                    a = offs[currentI] + lens[currentI];

                    for (; (height > 1) || (width > 1);) {
                        height >>= 1; width >>= 1;
                         ++currentI;                       
                        offs[currentI] = offs[currentI - 1];
                        lens[currentI] = lens[currentI - 1];
                    }
                        

                    for (; ++currentI < 16;)offs[currentI] = lens[currentI] = 0;

                    if (lpVtbl->sbWrite(saveParam->sb, saveParam->sb->current, header, a) == NULL)
                        saveParam->errorCode = IMAGINEERROR_WRITEERROR;
                }
                else saveParam->errorCode = IMAGINEERROR_OUTOFMEMORY;
            }
        }
        else saveParam->errorCode = IMAGINEERROR_COLORNOTSUPPORTED;
    }
    return result;
}

LPCTSTR savecheckbox[9] = { "BLP1 JPEG base","BLP1 JPEG progressive","BLP1 Paletted",
    "BLP2 JPEG base","BLP2 JPEG progressive","BLP2 Paletted","BLP2 DirX Compress","BLP2 Pal 4bit alpha","BLP2 Pal 1bit alpha\0" };
LPCTSTR savecheckboxU[9] = { (LPCTSTR)UNICODE_TEXT("BLP1 JPEG base"),(LPCTSTR)UNICODE_TEXT("BLP1 JPEG progressive"),(LPCTSTR)UNICODE_TEXT("BLP1 Paletted"),
    (LPCTSTR)UNICODE_TEXT("BLP2 JPEG base"),(LPCTSTR)UNICODE_TEXT("BLP2 JPEG progressive"),(LPCTSTR)UNICODE_TEXT("BLP2 Paletted"),(LPCTSTR)UNICODE_TEXT("BLP2 DirX Compress"),
    (LPCTSTR)UNICODE_TEXT("BLP2 Pal 4bit alpha"),(LPCTSTR)UNICODE_TEXT("BLP2 Pal 1bit alpha\0") };
LPCTSTR palettecheckbox[5] = { "NeuQuant 100","NeuQuant 128","Dennis Lee3 fast","Dennis Lee3 slow","Mediancut\0"};
LPCTSTR palettecheckboxU[5] = { (LPCTSTR)UNICODE_TEXT("NeuQuant 100"), (LPCTSTR)UNICODE_TEXT("NeuQuant 128"),(LPCTSTR)UNICODE_TEXT("Dennis Lee3 fast"),(LPCTSTR)UNICODE_TEXT("Dennis Lee3 slow"),(LPCTSTR)UNICODE_TEXT("Mediancut\0") };

LPCTSTR lppaletted = "To paletted";
LPCTSTR lppalettedU = (LPCTSTR)UNICODE_TEXT("To paletted");
LPCTSTR lpquality = "Quality jpg";
LPCTSTR lpqualityU = (LPCTSTR)UNICODE_TEXT("Quality jpg");
LPCTSTR lpformat = "Format";
LPCTSTR lpformatU= (LPCTSTR)UNICODE_TEXT("Format");
LPCTSTR lpMMipmaps = "Max mipmaps";
LPCTSTR lpMMipmapsU = (LPCTSTR)UNICODE_TEXT("Max mipmaps");

static IMAGINEOPTIONITEMSCROLLBAR Quality = { 1,100 };
static IMAGINEOPTIONITEMCOMBOBOX BLPType = { 9,savecheckbox };
static IMAGINEOPTIONITEMCOMBOBOX PalettedVariant = { 5,palettecheckbox };

static IMAGINEOPTIONITEMSCROLLBAR MMipmaps = { 1,16 };

static IMAGINEOPTIONITEM items[4] = { { IMAGINEOPTIONITEM_SCROLLBAR,lpquality,80,(IMAGINEOPTIONITEMCUSTOM*)&Quality },
    { IMAGINEOPTIONITEM_COMBOBOX,lpformat,1,(IMAGINEOPTIONITEMCUSTOM*)&BLPType },
    { IMAGINEOPTIONITEM_COMBOBOX,lppaletted,0,(IMAGINEOPTIONITEMCUSTOM*)&PalettedVariant },
    { IMAGINEOPTIONITEM_SCROLLBAR,lpMMipmaps,16,(IMAGINEOPTIONITEMCUSTOM*)&MMipmaps }
};
static IMAGINEOPTION saveoptions = { 4,items };
// file information (ANSI version)
static const IMAGINEFILEINFOITEM fileInfoItemA =
{
    checkFile,
    loadFile,
    saveFile,
    FILETYPE_BLP,
    EXTENSION_BLP,
    NULL,
    &saveoptions
};

// file information (UNICODE version)
static const IMAGINEFILEINFOITEM fileInfoItemW =
{
    checkFile,
    loadFile,
    saveFile,
    (LPCTSTR)UNICODE_TEXT(FILETYPE_BLP),
    (LPCTSTR)UNICODE_TEXT(EXTENSION_BLP),
    NULL,
    &saveoptions
};

const IMAGINEFILEINFOITEM *IMAGINEAPI IPSGetFileInfoA(void)
{
    items[0].text = lpquality;
    items[1].text = lpformat;
    items[2].text = lppaletted;
    items[3].text = lpMMipmaps;
    BLPType.itemString = savecheckbox;
    PalettedVariant.itemString = palettecheckbox;
    return &fileInfoItemA;
}

const IMAGINEFILEINFOITEM *IMAGINEAPI IPSGetFileInfoW(void)
{
    items[0].text = lpqualityU;
    items[1].text = lpformatU;
    items[2].text = lppalettedU;
    items[3].text = lpMMipmapsU;
    BLPType.itemString = savecheckboxU;
    PalettedVariant.itemString = palettecheckboxU;
    return &fileInfoItemW;
}
