#include "BLPPhotoshop.h"
#include "MCCQ.h"
#include "blpjpeg.h"
#include "dxt.h"

extern unsigned long dl3quantfast(unsigned char *inbuf, unsigned long pixels, unsigned char* userpal, unsigned long colorDepth);
extern unsigned long dl3quantslow(unsigned char *inbuf, unsigned long pixels, unsigned char* userpal, unsigned long colorDepth);
extern void neuquantcq(unsigned char *thepic, unsigned long pixels, unsigned char* pal, unsigned long colorDepth, unsigned long cycles);

#ifdef _M_X64
LPCWSTR szAboutstr = L"Photoshop file plugin\n for Blizzard image format - BLP.\n Version: 1.5 64-bit. ";
#else
LPCWSTR szAboutstr = L"Photoshop file plugin\n for Blizzard image format - BLP.\n Version: 1.5 32-bit. ";
#endif

//-------------------------------------------------------------------------------
//	Globals -- Define global variables for plug-in scope.
//-------------------------------------------------------------------------------

static SPBasicSuite * sSPBasic = NULL;
static SPPluginRef gPluginRef = NULL;
static PSHandleSuite2* sPSHandle;
static Data * gData = NULL;
static FormatRecord * gFormatRecord = NULL;
static Handle* gDataHandle = NULL;
static int16 * gResult = NULL;
static HINSTANCE hDLLModule;
static float rgbtoxyz[256];




//-------------------------------------------------------------------------------
//	Prototypes
//-------------------------------------------------------------------------------
static void DoReadPrepare(void);
static void DoReadStart(void);
static void DoReadContinue(void);
static void DoReadFinish(void);
static void DoOptionsPrepare(void);
static void DoOptionsStart(void);
static void DoEstimatePrepare(void);
static void DoEstimateStart(void);
static void DoWritePrepare(void);
static void DoWriteStart(void);
static void DoWriteContinue(void);
static void DoWriteFinish(void);
static void DoFilterFile(void);
static void DoAbout(SPPluginRef plugin, int dialogID);








BOOL APIENTRY DllMain(HANDLE hModule,    DWORD ,    LPVOID )
{
    hDLLModule = static_cast<HINSTANCE>(hModule);
    return  true;
}
//-------------------------------------------------------------------------------
//
//	PluginMain / main
//
//	All calls to the plug-in module come through this routine.
//	It must be placed first in the resource.  To achieve this,
//	most development systems require this be the first routine
//	in the source.
//
//	The entrypoint will be "pascal void" for Macintosh,
//	"void" for Windows.
//
//	Inputs:
//		const int16 selector						Host provides selector indicating
//													what command to do.
//
//		FormatRecord * formatParamBlock				Host provides pointer to parameter
//													block containing pertinent data
//													and callbacks from the host.
//													See PIFormat.h.
//
//	Outputs:
//		FormatRecord * formatParamBlock				Host provides pointer to parameter
//													block containing pertinent data
//													and callbacks from the host.
//													See PIFormat.h.
//
//		void * data									Use this to store a handle or pointer to our
//													global parameters structure, which
//													is maintained by the host between
//													calls to the plug-in.
//
//		int16 * result								Return error result or noErr.  Some
//													errors are handled by the host, some
//													are silent, and some you must handle.
//													See PIGeneral.h.
//
//-------------------------------------------------------------------------------

DLLExport MACPASCAL void PluginMain(const int16 selector,
    FormatRecordPtr formatParamBlock, intptr_t * data,
    int16 * result)
{
    try {

        //---------------------------------------------------------------------------

        if (selector == formatSelectorAbout) {
            //AboutRecordPtr aboutRecord = reinterpret_cast<AboutRecordPtr>(formatParamBlock);
            //sSPBasic = aboutRecord->sSPBasic;
            //gPluginRef = reinterpret_cast<SPPluginRef>(aboutRecord->plugInRef);
#if Macintosh
#else
            MessageBoxW(GetActiveWindow(), szAboutstr, L"About plugin", MB_OK);
#endif

            *gResult = noErr;
            return;
        }
        else { // do the rest of the process as normal:
            gFormatRecord = reinterpret_cast<FormatRecordPtr>(formatParamBlock);
            gPluginRef = reinterpret_cast<SPPluginRef>(gFormatRecord->plugInRef);
            gResult = result;
            gDataHandle = reinterpret_cast<Handle*>(data);

            sSPBasic = gFormatRecord->sSPBasic;
            
            if (gFormatRecord->resourceProcs->countProc == NULL ||
                gFormatRecord->resourceProcs->getProc == NULL ||
                gFormatRecord->resourceProcs->addProc == NULL ||
                gFormatRecord->advanceState == NULL) {
                *gResult = errPlugInHostInsufficient;
                return;
            }

            // new for Photoshop 8, big documents, rows and columns are now > 30000 pixels
            if (gFormatRecord->HostSupports32BitCoordinates)
                gFormatRecord->PluginUsing32BitCoordinates = true;

            //-----------------------------------------------------------------------
            //	(3) Allocate and initalize globals.
            //
            //-----------------------------------------------------------------------

            if (selector== formatSelectorReadPrepare|| selector == formatSelectorEstimateStart) {
                sSPBasic->AcquireSuite(kPSHandleSuite, kPSHandleSuiteVersion2, (const void**)&sPSHandle);                
                if (*gDataHandle) {
                    /*Boolean oldLock = FALSE;
                    sPSHandle->SetLock(*gDataHandle, false,
                        reinterpret_cast<Ptr *>(&gData), &oldLock);
                    sPSHandle->Dispose(*gDataHandle);*/
                    sPSHandle->SetSize(*gDataHandle, sizeof(Data));
                }
                else {
                    // Create a handle to our Data structure. Photoshop will take ownership of this
                    // handle and delete it when necessary.


                    Handle h = sPSHandle->New(sizeof(Data));
                    if (h != NULL)
                        *gDataHandle = h;
                    else
                        *gResult = memFullErr;
                }

                
            }

            if (*gResult == noErr&&(selector== formatSelectorReadPrepare||selector== formatSelectorReadStart||
                selector== formatSelectorReadContinue||selector== formatSelectorReadFinish||
                selector== formatSelectorEstimateStart|| selector == formatSelectorWritePrepare||
                selector == formatSelectorWriteStart || selector == formatSelectorWriteContinue 
                )) {
                // Lock the handles and get the pointers for gData
                
                Boolean oldLock = FALSE;
                sPSHandle->SetLock(reinterpret_cast<Handle>(*gDataHandle), true, reinterpret_cast<Ptr *>(&gData), &oldLock);
                if (gData == NULL) *gResult = memFullErr;
            }
            if (*gResult == noErr) {
                //-----------------------------------------------------------------------
                //	(4) Dispatch selector.
                //-----------------------------------------------------------------------
                switch (selector) {
                case formatSelectorReadPrepare:
                    DoReadPrepare();
                    break;
                case formatSelectorReadStart:
                    DoReadStart();
                    break;
                case formatSelectorReadContinue:
                    DoReadContinue();
                    break;
                case formatSelectorReadFinish:
                    //DoReadFinish();
                    break;

                case formatSelectorOptionsPrepare:
                    DoOptionsPrepare();
                    break;
                case formatSelectorOptionsStart:
                    DoOptionsStart();
                    break;
                case formatSelectorOptionsContinue:
                    //DoOptionsContinue();
                    break;
                case formatSelectorOptionsFinish:
                    //DoOptionsFinish();
                    break;

                case formatSelectorEstimatePrepare:
                    DoEstimatePrepare();
                    break;
                case formatSelectorEstimateStart:
                    DoEstimateStart();
                    break;
                case formatSelectorEstimateContinue:
                    //DoEstimateContinue();
                    break;
                case formatSelectorEstimateFinish:
                    //DoEstimateFinish();
                    break;

                case formatSelectorWritePrepare:
                    DoWritePrepare();
                    break;
                case formatSelectorWriteStart:
                    DoWriteStart();
                    break;
                case formatSelectorWriteContinue:
                    DoWriteContinue();
                    break;
                case formatSelectorWriteFinish:
                    //DoWriteFinish();
                    break;

                case formatSelectorFilterFile:
                    DoFilterFile();
                    break;
                }
            }
            	

            

        } 

          // release any suites that we may have acquired
        if (selector == formatSelectorAbout ||
            selector == formatSelectorWriteFinish ||
            selector == formatSelectorReadFinish ||
            selector == formatSelectorFilterFile ||
            *gResult != noErr) {
            Boolean oldLock = FALSE;
            if (*gDataHandle) {
                
                if (gData->hbuf) {
                    sPSHandle->SetLock(gData->hbuf, false,
                        reinterpret_cast<Ptr *>(&gData->buf), &oldLock);
                    sPSHandle->Dispose(gData->hbuf);
                }
                sPSHandle->SetLock(*gDataHandle, false,
                    reinterpret_cast<Ptr *>(&gData), &oldLock);

                sPSHandle->Dispose(*gDataHandle);

                *gDataHandle = NULL;
                gDataHandle = NULL;
            }
            
            sSPBasic->ReleaseSuite(kPSHandleSuite, kPSHandleSuiteVersion2);
            sSPBasic = NULL;
        }

    } // end try

    catch (...) {
        if (NULL != result)*result = -1;
    }

} // end PluginMain

static void ReadFl(void* buf, unsigned long count, unsigned long pos)
{
    unsigned long readCount=count;
#if Macintosh
    FSSetForkPosition(gFormatRecord->dataFork, 0, pos);
    FSReadFork(gFormatRecord->dataFork, fsAtMark, 0, count, buf, &readCount);
    if (readCount != count)
        *gResult = readErr;
#else
    if (SetFilePointer((HANDLE)gFormatRecord->dataFork, pos, NULL, FILE_BEGIN)!= pos) { *gResult = writErr; return; }
    else *gResult = noErr;

    if (!ReadFile((HANDLE)gFormatRecord->dataFork, buf, count, &readCount, NULL)||count != readCount) {
        *gResult = readErr;
    }
#endif
}
static void WriteFl(void* buf, unsigned long count, unsigned long pos)
{
    unsigned long writeCount = count;
#if Macintosh
    FSSetForkPosition(gFormatRecord->dataFork, 0, pos);
    OSErr err = FSWriteFork(gFormatRecord->dataFork, fsAtMark | noCacheMask, 0, count, buf, &readCount);
    if (noErr != err)*gResult = err;
    if (readCount != count)
        *gResult = readErr;
#else
    if (SetFilePointer((HANDLE)gFormatRecord->dataFork, pos, NULL, FILE_BEGIN) != pos) { *gResult = writErr; return; }
    else *gResult = noErr;

    if (!WriteFile((HANDLE)gFormatRecord->dataFork, buf, count, &writeCount, NULL) || count != writeCount) {
        *gResult = readErr;
    }
#endif
}

FormatRecord s;

static void DoFilterFile(void)
{
    BLPHeader header;

    ReadFl(&header, sizeof(BLPHeader),0);
    
    if (gResult == noErr) {
        if ((header.Signature == BLP0 || header.Signature == BLP1 || header.Signature == BLP2) &&
            header.Height < 32769 && header.Height&&header.Width&&header.Width < 32769 && header.PixelCompression < 2)
            return;
        *gResult = formatCannotRead;
    }

}

static void DoReadPrepare()
{
    BLPHeader header;
    ReadFl(&header, sizeof(BLPHeader), 0);
    uint32 ImgHeaderof;
    if (*gResult == noErr) {
        if (header.Signature == BLP2) {
            gData->CurOfs = header.HeaderEnd.blp2.offsets[0];
            gData->CurLen = header.HeaderEnd.blp2.lengths[0];
            if (gFormatRecord->openForPreview) {
                uint32 m = header.Height;
                if (m < header.Width)m = header.Width;
                uint32 i;
                for (i = 0; i < 15 && m>127 && header.HeaderEnd.blp2.offsets[i + 1]; i++)m >>= 1;
                gData->CurOfs = header.HeaderEnd.blp2.offsets[i];
                gData->CurLen = header.HeaderEnd.blp2.lengths[i];
                header.Width >>= i;
                header.Height >>= i;
                if (!header.Width)header.Width = 1;
                if (!header.Height)header.Height = 1;
            }
            ImgHeaderof = 0x94U;
        }
        else {
            gData->CurOfs = header.HeaderEnd.blp1.offsets[0];
            gData->CurLen = header.HeaderEnd.blp1.lengths[0];
            if (gFormatRecord->openForPreview) {
                uint32 m = header.Height;
                if (m < header.Width)m = header.Width;
                uint32 i;
                for (i = 0; i < 15 && m>127 && header.HeaderEnd.blp1.offsets[i + 1]; i++)m>>=1;
                gData->CurOfs = header.HeaderEnd.blp1.offsets[i];
                gData->CurLen = header.HeaderEnd.blp1.lengths[i];
                header.Width >>= i;
                header.Height >>= i;
                if (!header.Width)header.Width = 1;
                if (!header.Height)header.Height = 1;
            }
            ImgHeaderof = 0x9cU;
        }
        gData->Height = header.Height;
        gData->Width = header.Width;
        if ((header.Signature == BLP0 || header.Signature == BLP1 || header.Signature == BLP2) &&
            gData->CurLen&&header.Height < 32769 && header.Height&&header.Width&&header.Width < 32769 && header.PixelCompression < 2) {
            gFormatRecord->maxData = 2097152;
            if (header.PixelCompression) {
                if (ImgHeaderof == 0x9CU ) {
                    if(!header.Flags)gData->Type = 2;
                    else gData->Type = 0x802;
                }
                else if ((header.Flags & 0xff) == PALETTED) {
                    gData->Type = (header.Flags & 0xf00)+2;
                    if (gData->Type != 2 && gData->Type != 0x102 && gData->Type != 0x202 && gData->Type != 0x402) gData->Type = 0x802;
                }
                else {
                    if ((header.Flags & 0xff) == DXTCOMPRESSED) {
                        if ((header.Flags & 0xff0000) == DXT3) gData->Type = 0x803;
                        else if ((header.Flags & 0xff0000) == DXT5) gData->Type = 0x805;
                        else gData->Type = 1;
                    }
                    else if ((header.Flags & 0xff) == UNCOMPRESSED)gData->Type = 4+(header.Flags & 0xf00);
                }
                if ((gData->Type & 0xff) == 2) {
                    ReadFl(gData->Palette, 1024, ImgHeaderof);
                    for (uint32 i = 0; i < 256; i++) {
                        uint8 b = gData->Palette[i * 4];
                        gData->Palette[i * 4] = gData->Palette[i * 4 + 2];
                        gData->Palette[i * 4 + 2] = b;
                    }
                }
            }
            else {
                if(header.Flags&&ImgHeaderof == 0x9CU)gData->Type = 0x10806;
                else if(ImgHeaderof == 0x94U)gData->Type = 6+ (header.Flags & 0xf00);
                else gData->Type = 0x10006;
                gFormatRecord->maxData = 1048576 + gData->CurLen +gData->CurOfs + gData->Width*gData->Height * 4 + sizeof(blpjpeg::jpegDecompress)+ ((gData->Height + 7) >> 3)*((gData->Width + 7) >> 3) * 512;
            }
            gData->CurLine = 0;
        }
        else *gResult = formatCannotRead;
    }
}
static void DoReadStart(void)
{
    if ((gData->Type & 0xff) == 2) { 
        gFormatRecord->imageMode = plugInModeIndexedColor;
        gFormatRecord->planes = 1;
        for (uint32 i = 0; i < 256; i++) {
            gFormatRecord->redLUT[i] = gData->Palette[i * 4];
            gFormatRecord->greenLUT[i] = gData->Palette[i * 4+1];
            gFormatRecord->blueLUT[i] = gData->Palette[i * 4+2];
        }
        gFormatRecord->lutCount = 256;
    }
    else { 
        gFormatRecord->imageMode = plugInModeRGBColor; 
        gFormatRecord->planes = 3;
    }
    if (gFormatRecord->HostSupports32BitCoordinates) {
        gFormatRecord->imageSize32.v = gData->Height;
        gFormatRecord->imageSize32.h = gData->Width;
    }
    else {
        gFormatRecord->imageSize.v=(int16)gData->Height;
        gFormatRecord->imageSize.h = (int16)gData->Width;
    }
    gFormatRecord->depth = 8;
    if (gData->Type & 0xff00) {
        gFormatRecord->planes++;
        gFormatRecord->transparencyPlane = -1;
        
        if (!gFormatRecord->openForPreview&&
            IDNO==MessageBoxW(GetActiveWindow(), L"Make alpha discrete channel?", L"BLP open option", MB_YESNO))gFormatRecord->transparencyPlane = gFormatRecord->planes-1;
        gFormatRecord->transparencyMatting = 0;
    }
    
    gFormatRecord->imageHRes = gFormatRecord->imageVRes = 72;
    
    uint32 a= 1048576;
    if ((gData->Type & 0xff) == 6)
        a = gData->CurLen+gData->CurOfs + gData->Width*gData->Height * 4 +
        sizeof(blpjpeg::jpegDecompress) + ((gData->Height + 7)>>3)*((gData->Width + 7)>>3) * 512;
    gData->hbuf = sPSHandle->New(a);
    if (gData->hbuf != NULL) {
        Boolean oldLock = FALSE;
        sPSHandle->SetLock(gData->hbuf, true, reinterpret_cast<Ptr *>(&(gData->buf)), &oldLock);
        if (gData->buf == NULL)*gResult = memFullErr;
        else DoReadContinue();
    }
    else
        *gResult = memFullErr;
}
static void DoReadContinue(void) {
    uint8* dest;
    uint32 a;
    a = (uint8)gData->Type;
    if (gData->CurLine >= gData->Height) {
        if ((gData->Type & 0xffff00) && a == 2) {
            a = 8; gData->CurLine = 0; gData->Type += 6;
        }
        else { gFormatRecord->data = NULL; return; }
    }
    gFormatRecord->planeBytes = 1;
    if (gFormatRecord->HostSupports32BitCoordinates) {
        gFormatRecord->theRect32.left = 0;
        gFormatRecord->theRect32.right = gData->Width;
        gFormatRecord->theRect32.top = gData->CurLine;
    }
    else {
        gFormatRecord->theRect.left = 0;
        gFormatRecord->theRect.right = (int16)gData->Width;
        gFormatRecord->theRect.top = (int16)gData->CurLine;
    }
    gFormatRecord->loPlane = 0;
    gFormatRecord->data = gData->buf;
    gFormatRecord->colBytes = 4;
    if (gData->Type & 0xff00) { 
        gFormatRecord->hiPlane = 3;
        
    }
    else { 
        gFormatRecord->hiPlane = 2;
    }
    gFormatRecord->rowBytes = gFormatRecord->colBytes*gData->Width;

    if (a == 6) {
        //jpg
        gFormatRecord->planeMap[0] = 2; gFormatRecord->planeMap[2] = 0;
        uint32 l,of;
        if (gData->Type & 0xff0000)of = 0x9c;
        else of = 0x94;
        ReadFl(&l, 4, of);
        of += 4;
        if (*gResult == noErr) {
            dest = gData->buf + 4 * gData->Height*gData->Width;
            if(l)ReadFl(dest, l, of);
            ReadFl(dest+l, gData->CurLen, gData->CurOfs);
            if (*gResult == noErr) {
                blpjpeg::jpegDecompress* pjpegDecompress = (blpjpeg::jpegDecompress*)(dest + l + gData->CurLen);
                pjpegDecompress->bitmap = gData->buf;
                pjpegDecompress->length = l+ gData->CurLen;
                pjpegDecompress->source = dest;
                pjpegDecompress->height = gData->Height;
                pjpegDecompress->width = gData->Width;
                pjpegDecompress->dehuff = (short*)(pjpegDecompress + 1);
                pjpegDecompress->callback = NULL;
                blpjpeg::blpjpegDecompress(pjpegDecompress);

                for (uint32 line = (gData->Height >> 1)-1;;) {
                    uint32 *l1, *l2;
                    l1 = (uint32*)(gData->buf + line*gData->Width * 4);
                    l2 = (uint32*)(gData->buf + (gData->Height-line-1)*gData->Width * 4);
                    for (uint32 x = 0; x < gData->Width; x++) {
                        a = l2[x];
                        l2[x] = l1[x];
                        l1[x] = a;
                    }
                    if (!line--)break;
                }
                gData->CurLine = gData->Height;
            }
        }
    }
    else {
        uint32 lines;

        if (a == 2) {//paletted
            gFormatRecord->hiPlane = 0;
            gFormatRecord->colBytes = 1;
            gFormatRecord->rowBytes = gData->Width;
            lines = 1048576 / gData->Width;
            a = gData->Height - gData->CurLine;
            if (lines > a)lines = a;
            gData->CurLine += lines;
            a = lines*gData->Width;
            ReadFl(gData->buf, a, gData->CurOfs);
            gData->CurOfs += a;
        }
        else if(a==8){//paletted alpha
            gFormatRecord->loPlane = 1;
            gFormatRecord->hiPlane = 1;
            gFormatRecord->colBytes = 1;
            gFormatRecord->rowBytes = gData->Width;
            lines = 1048576 / gData->Width;
            a = gData->Height - gData->CurLine;
            if (lines > a)lines = a;
            if(lines>7)lines &= 0xfffffff8;
            gData->CurLine += lines;
            a = lines*gData->Width;
            uint32 af=gData->Type&0xffff00,a1;
            if (af == 0x100)a1 = (a + 7) / 8;
            else if (af == 0x200)a1 = (a + 3) / 4;
            else if (af == 0x400)a1 = (a + 1) / 2;
            else a1 = a;
            ReadFl(gData->buf, a1, gData->CurOfs);
            gData->CurOfs += a1;
            a1 = gData->Type & 0xff00;
            uint8 bof = 0,b;
            if(af==0x100){
                b = a & 7;
                if (b)bof = 1 << (b-1);
                b = gData->buf[a >> 3];
                for (uint32 x = a; x--;) {
                    if (!bof) {
                        bof = 0x80;
                        b = gData->buf[x >> 3];
                    }
                    if (b&bof)gData->buf[x] = 0xff;
                    else gData->buf[x] = 0;
                    bof >>= 1;
                }
            }
            else if(af==0x200){
                if ((a & 3) == 1)bof = 2;
                else if ((a & 3) == 2)bof = 8;
                else if ((a & 3) == 3)bof = 0x20;
                b = gData->buf[a >> 2];
                for (uint32 x = a; x--;) {
                    if (!bof) {
                        bof = 0x80;
                        b = gData->buf[x >> 2];
                    }
                    uint8 b1 = 0;
                    if (b&bof)b1 = 0xaa;
                    bof >>= 1;
                    if (b&bof)b1 |= 0x55;
                    gData->buf[x] = b1;
                    bof >>= 1;
                }
            }
            else if(af==0x400){
                if (a & 1) {
                    bof = 1;
                    b = gData->buf[a >> 1];
                }
                for (uint32 x = a; x--;) {
                    uint8 b1;
                    if (!bof) {
                        b = gData->buf[x >> 1];
                        b1= b>>4;
                    }
                    else {
                        b1 = b & 0xf;
                    }
                    bof = !bof;
                    gData->buf[x] = b1*0x11;
                }
            }
        }
        else if(a==4){//uncompressed
            gFormatRecord->planeMap[0] = 2; gFormatRecord->planeMap[2] = 0;
            lines = 262144 / gData->Width;
            a = gData->Height - gData->CurLine;
            if (lines > a)lines = a;
            gData->CurLine += lines;
            ReadFl(gData->buf, 4*lines*gData->Width, gData->CurOfs);
            gData->CurOfs += 4 * lines*gData->Width;
        }
        else {//dxt
            uint32 l = ((gData->Width + 3) >> 2) << 3;
            if (a != 1)l += l;
            gData->CurLine += 4;
            uint32 *sourced = (uint32*)(gData->buf+16*gData->Width);
            ReadFl(sourced, l, gData->CurOfs);
            gData->CurOfs += l;
            if (gData->CurLine > gData->Height)gData->CurLine = gData->Height;
            
            uint8 unp[64];
            for (uint32 x = 0; x < gData->Width; x++) {
                uint32 t = (x & 3) << 2;
                if (!t) {
                    dxt::DecompressDxt(unp, (uint8*)sourced, a);
                    if (a == 1)sourced += 2;
                    else sourced += 4;
                }
                uint32 *destd = (uint32*)(gData->buf + x * 4);
                *destd = *(uint32*)(unp + t);
                destd += gData->Width;
                *destd = *(uint32*)(unp + t + 16);
                destd += gData->Width;
                *destd = *(uint32*)(unp + t + 32);
                destd += gData->Width;
                *destd = *(uint32*)(unp + t + 48);
            }
        }
    }
    if (gFormatRecord->HostSupports32BitCoordinates)
        gFormatRecord->theRect32.bottom = gData->CurLine;
    else
        gFormatRecord->theRect.bottom = (int16)gData->CurLine;

}
static void DoReadFinish(void)
{
}

static void setScroll(HWND hdlg, uint32 n)
{
    SCROLLINFO si;
    si.cbSize = sizeof(SCROLLINFO);
    si.fMask = SIF_POS;
    si.nPos = n;
    SetScrollInfo(GetDlgItem(hdlg, IDC_SCROLLBAR1), SB_CTL, &si, TRUE);
    WCHAR ts[4];
    ts[1] = ts[2] = ts[3] = 0;
    if (n == 100) { ts[0] = L'1'; ts[1] = L'0'; ts[2] = L'0'; }

    else {
        wchar_t t = (wchar_t)(n / 10);
        if (t) { ts[0] = t + L'0'; ts[1] = (wchar_t)n + L'0' - t * 10; }
        else ts[0] = (wchar_t)n + L'0';
    }
    SetDlgItemTextW(hdlg, IDC_STATICQ, ts);
}
INT_PTR CALLBACK DlgProcgsp(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch (msg) {
    case WM_INITDIALOG:
    {
        LPCTSTR dlgstr[12] = { L"BLP1 JPEG",L"BLP1 Paletted", L"BLP2 JPEG",L"BLP2 Paletted 8-bit alpha",L"BLP2 DirX Compress",L"BLP2 Paletted 4-bit alpha",L"BLP2 Paletted 1-bit alpha",
            L"NeuQuant 100",L"NeuQuant 128",L"Dennis Lee3 fast",L"Dennis Lee3 slow",L"Mediancut" };
        HWND hwndl = GetDlgItem(hdlg, IDC_COMBOBOXEX1);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[0]);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[1]);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[2]);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[3]);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[4]);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[5]);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[6]);
        SendMessageW(hwndl, CB_SETCURSEL, 0, 0);
        hwndl = GetDlgItem(hdlg, IDC_COMBOBOXEX2);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[7]);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[8]);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[9]);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[10]);
        SendMessageW(hwndl, CB_ADDSTRING, 0, (LPARAM)dlgstr[11]);
        SendMessageW(hwndl, CB_SETCURSEL, 0, 0);
        SetDlgItemTextW(hdlg, IDC_STATICQ, L"100");
        SetDlgItemTextW(hdlg, IDC_EDIT2, L"16");
        hwndl = GetDlgItem(hdlg, IDC_SPIN1);
        SendMessageW(hwndl, UDM_SETRANGE, 0, MAKELPARAM(16,1));
        SendMessageW(hwndl, UDM_SETPOS, 0, MAKELPARAM(16, 0));
        hwndl = GetDlgItem(hdlg, IDC_SCROLLBAR1);
        SCROLLINFO si;
        si.cbSize = sizeof(SCROLLINFO);
        si.fMask = SIF_ALL;
        si.nMin = 1;
        si.nMax = 100;
        si.nPage = 1;
        si.nPos = 100;
        SetScrollInfo(hwndl, SB_CTL, &si, TRUE);
        return TRUE;
    }
    break;

    case WM_HSCROLL:
    {
        if (LOWORD(wParam) == SB_THUMBTRACK) {
            uint32 a = HIWORD(wParam);
            if (a < 1)a = 1;
            else if (a>100)a = 100;
            setScroll(hdlg, a);
        }
        else if (LOWORD(wParam) == SB_LINEUP) {
            SCROLLINFO si;
            si.cbSize = sizeof(SCROLLINFO);
            si.fMask = SIF_POS;
            GetScrollInfo(GetDlgItem(hdlg, IDC_SCROLLBAR1), SB_CTL, &si);
            if (si.nPos > 1)si.nPos--;
            setScroll(hdlg, si.nPos);
        }
        else if (LOWORD(wParam) == SB_LINEDOWN) {
            SCROLLINFO si;
            si.cbSize = sizeof(SCROLLINFO);
            si.fMask = SIF_POS;
            GetScrollInfo(GetDlgItem(hdlg, IDC_SCROLLBAR1), SB_CTL, &si);
            if (si.nPos < 100)si.nPos++;
            setScroll(hdlg, si.nPos);
        }
        return TRUE;
    }
        break;
    case WM_COMMAND:
        if (LOWORD(wParam) == IDOK) {
            INT_PTR a, a1;
            SCROLLINFO si;
            si.cbSize = sizeof(SCROLLINFO);
            si.fMask = SIF_POS;
            GetScrollInfo(GetDlgItem(hdlg, IDC_SCROLLBAR1), SB_CTL, &si);
            a = si.nPos;
            HWND hwndl = GetDlgItem(hdlg, IDC_COMBOBOXEX1);
            a1 = SendMessageW(hwndl, CB_GETCURSEL, 0, 0);
            a |= a1 << 8;
            if (a1 == 0 || a1 == 2) {
                if (IsDlgButtonChecked(hdlg, IDC_CHECK1))a |= 0x1000;
            }
            hwndl = GetDlgItem(hdlg, IDC_COMBOBOXEX2);
            a1 = SendMessageW(hwndl, CB_GETCURSEL, 0, 0);
            a |= a1 << 16;
            a1= SendMessageW(GetDlgItem(hdlg, IDC_SPIN1), UDM_GETPOS, 0, 0);
            a |= a1 << 24;
            EndDialog(hdlg, a);
            return TRUE;
        }
        else if (LOWORD(wParam) == IDC_EDIT2&&HIWORD(wParam)==EN_CHANGE) {
            BOOL aa;
            UINT a = GetDlgItemInt(hdlg, IDC_EDIT2, &aa, false);
            if(a<1)SetDlgItemInt(hdlg, IDC_EDIT2, 1, false);
            if(a>16)SetDlgItemInt(hdlg, IDC_EDIT2, 16, false);
        }
        break;
    }
    return FALSE;
}

static void DoOptionsPrepare(void) {
    gFormatRecord->maxData = 1048576;
}
static void DoOptionsStart(void) {
    
    gFormatRecord->data = NULL;
}

static void DoEstimatePrepare(void) {
    gFormatRecord->maxData = 1048576;
}
static void DoEstimateStart(void) {
    uint32 a = gFormatRecord->imageSize32.h*gFormatRecord->imageSize32.v;
    gFormatRecord->minDataBytes = a+1024;
    gFormatRecord->maxDataBytes = a*8;
    gFormatRecord->data = NULL;
    if (gFormatRecord->HostSupports32BitCoordinates) {
        gData->Height = gFormatRecord->imageSize32.v;
        gData->Width = gFormatRecord->imageSize32.h;
    }
    else {
        gData->Height = gFormatRecord->imageSize.v;
        gData->Width = gFormatRecord->imageSize.h;
    }
    a = gFormatRecord->imageMode;
    if (a > 3)*gResult = formatCannotRead;
    gData->saveparam = (uint32)DialogBoxParamW(hDLLModule, MAKEINTRESOURCE(IDD_DIALOG1), GetActiveWindow(), &DlgProcgsp, 0L);
}

static void DoWritePrepare(void) {
    gFormatRecord->maxData = gData->Height*gData->Width*12+0x2000;
    uint32 savef = gData->saveparam & 0xf00;
    if (savef == 0 || savef == 0x200)gFormatRecord->maxData += sizeof(blpjpeg::jpegCompress) + ((gData->Height + 7) >> 3)*((gData->Width + 7) >> 3) * 512;
}
static void DoWriteStart(void) {

    uint32 a;
    uint32 savef = gData->saveparam & 0xf00;
    a = gData->Height*gData->Width * 12 + 0x2000;
    if (savef == 0 || savef == 0x200)a += sizeof(blpjpeg::jpegCompress) + ((gData->Height + 7) >> 3)*((gData->Width + 7) >> 3) * 512;
    gData->hbuf = sPSHandle->New(a);
    Boolean oldLock = FALSE;
    sPSHandle->SetLock(gData->hbuf, true, reinterpret_cast<Ptr *>(&gData->buf), &oldLock);
    if (gData->buf == NULL)*gResult = memFullErr;
    else {
        if (gFormatRecord->HostSupports32BitCoordinates) {
            gFormatRecord->theRect32.bottom = gFormatRecord->imageSize32.v;
            gFormatRecord->theRect32.left = 0;
            gFormatRecord->theRect32.right = gFormatRecord->imageSize32.h;
            gFormatRecord->theRect32.top = 0;
        }
        else {
            gFormatRecord->theRect.bottom = (int16)gFormatRecord->imageSize32.v;
            gFormatRecord->theRect.left = 0;
            gFormatRecord->theRect.right = (int16)gFormatRecord->imageSize32.h;
            gFormatRecord->theRect.top = 0;
        }
        gFormatRecord->loPlane = 0;
        gFormatRecord->hiPlane = gFormatRecord->planes - 1;
        if (gFormatRecord->planes == 3)gFormatRecord->colBytes = 4;
        else gFormatRecord->colBytes = gFormatRecord->planes;
        gFormatRecord->rowBytes = gFormatRecord->colBytes*gFormatRecord->imageSize32.h;
        gFormatRecord->planeBytes = 1;
        gFormatRecord->planeMap[0] = 0;
        gFormatRecord->planeMap[1] = 1;
        gFormatRecord->planeMap[2] = 2;
        gFormatRecord->planeMap[3] = 3;
        if (gFormatRecord->imageMode == plugInModeRGBColor)gFormatRecord->data = gData->buf;
        else gFormatRecord->data = gData->buf + 4 * gFormatRecord->imageSize32.v*gFormatRecord->imageSize32.h;
    }
}

static void DoWriteContinue(void) {
    uint32 *destd = (uint32*)gData->buf;
    uint32 c = gData->Height*gData->Width;
    uint8 *source= gData->buf + 4 * c;
    
    if(gFormatRecord->imageMode== plugInModeBitmap){
        for (uint32 y = 0; y < gData->Height; y++) {
            uint8 bof = 0;
            uint8 b;
            for (uint32 x = 0; x < gData->Width; x++) {
                if (!bof) {
                    bof = 0x80;
                    b = source[x>>3];
                }
                if (b&bof)destd[x] = 0xff000000;
                else destd[x] = 0xffffffff;
                bof >>= 1;
            }

            source += gData->Width;
            destd += gData->Width;
        }
        *(unsigned long long*)gData->Palette = 0xffffffffff000000;
    }
    else if (gFormatRecord->imageMode == plugInModeRGBColor) {
        if (gFormatRecord->planes == 3) {
            uint8* dest = gData->buf + 0x3;
            for (uint32 i = 0; i < c; i++)*(dest+i*4)=0xff;
        }
    }
    else {
        uint32 *pal = (uint32*)gData->Palette;
        if (gFormatRecord->imageMode == plugInModeGrayScale) {
            for (uint32 i = 0, a = 0xff000000; i < 256; i++, a += 0x010101)pal[i] = a;
        }
        else if (gFormatRecord->imageMode == plugInModeIndexedColor) {
            for (uint32 i = 0; i < 256; i++) {
                gData->Palette[i * 4] = gFormatRecord->blueLUT[i];
                gData->Palette[i * 4 + 1] = gFormatRecord->greenLUT[i];
                gData->Palette[i * 4 + 2] = gFormatRecord->redLUT[i];
                gData->Palette[i * 4 + 3] = 0xff;
            }
            if (gFormatRecord->planes == 1 && gFormatRecord->lutCount < 256 && gFormatRecord->transparentIndex)*(uint32*)(gData->Palette + gFormatRecord->transparentIndex * 4) = 0;
        }
        if (gFormatRecord->planes == 1) {
            for (uint32 i = 0; i < c; i++) destd[i] = pal[source[i]];
        }
        else {
            for (uint32 i = 0; i < c; i++) {
                uint32 a = 0xffffff&pal[*source];
                source++;
                a |= ((uint32)*source) << 24;
                source++;
                destd[i] = a;
            }
        }
    }
    gFormatRecord->data = 0;

    uint32 savef = gData->saveparam & 0xf00;
    if ((savef == 0x400) == (gFormatRecord->planes<3)) {
        source = gData->buf;
        for (uint32 i = c; i--;) {
            uint8 b = *source;
            *source = *(source + 2);
            *(source + 2) = b;
            source += 4;
        }
    }
    if (gFormatRecord->imageMode == plugInModeRGBColor && (savef == 0x100 || savef == 0x300 || savef == 0x500 || savef == 0x600)) {
        uint32 qw = gData->saveparam & 0xf0000;
        if (qw == 0x20000) {
            qw = dl3quantfast(gData->buf, c, gData->Palette, 4);
            if (qw<256)GetPaletteMCCQ(gData->Palette, (piped*)(gData->buf+4*c), gData->buf, c, 4);
        }
        else if (qw == 0x30000) {
            qw = dl3quantslow(gData->buf, c, gData->Palette, 4);
            if (qw<256)GetPaletteMCCQ(gData->Palette, (piped*)(gData->buf + 4 * c), gData->buf, c, 4);
        }
        else if (!qw && c>503)neuquantcq(gData->buf, c, gData->Palette, 4, 100);
        else if (qw == 0x10000 && c>503)neuquantcq(gData->buf, c, gData->Palette, 4, 128);
        else GetPaletteMCCQ(gData->Palette, (piped*)(gData->buf + 4 * c), gData->buf, c, 4);
    }
    


    
    uint8 *dest=gData->buf + 4 * c;
    if (savef == 0 || savef == 0x200)dest += sizeof(blpjpeg::jpegCompress) + ((gData->Height + 7)>>3)*((gData->Width + 7)>>3) * 512;
    //---------------alphaDepth-------------
    uint32 alphaDepth = 0;
    /*if (savef == 0x300) {
        if (gFormatRecord->imageMode == plugInModeIndexedColor&&gFormatRecord->planes == 1 && gFormatRecord->lutCount < 256 && gFormatRecord->transparentIndex)alphaDepth=1;
        else if (gFormatRecord->planes == 2 || gFormatRecord->planes == 4) {
            source = gData->buf + 0x3;
            for (uint32 i = c; i--; source+=4) {
                uint8 b = *source;
                if (!alphaDepth&&!b)alphaDepth++;
                else if ((b&0xf) != (b >> 4)) { alphaDepth = 8; break; }
                else if (b != 0xff)alphaDepth = 4;
            }
        }
    }
    else if (gFormatRecord->planes == 2 || gFormatRecord->planes == 4)alphaDepth = 8;*/
    if (gFormatRecord->planes == 2 || gFormatRecord->planes == 4) {
        alphaDepth = 8;
        if (savef == 0x500)alphaDepth = 4;
        if (savef == 0x600)alphaDepth = 1;
    }
    //----------------header-----------------
    BLPHeader* header = (BLPHeader*)dest;
    if (savef < 0x200)header->Signature = BLP1;
    else header->Signature = BLP2;
    header->Height = gData->Height;
    header->Width = gData->Width;
    if (savef == 0 || savef == 0x200)header->PixelCompression = 0;
    else header->PixelCompression = 1;

    header->Flags = 0;
    if (savef < 0x200) {
        dest += +0x9c;
        header->HeaderEnd.blp1.flags2 = 1;
        if (alphaDepth) {
            header->Flags = 8;
            header->HeaderEnd.blp1.alphaEncoding = 4;
        }
        else header->HeaderEnd.blp1.alphaEncoding = 5;
    }
    else {
        dest += 0x94;
        uint32 v = 0x1000000 + (alphaDepth << 8);
        if (savef == 0x300|| savef == 0x500|| savef == 0x600)v |= 0x80001;//blp2 paletted
        else if (savef == 0x400) {//dxt
            if (alphaDepth)v |= 0x70002;
            else v |= 0x2;
        }
        header->Flags = v;
    }

    uint32* offs, *lens;
    if (savef < 0x200) {
        offs = header->HeaderEnd.blp1.offsets;
        lens = header->HeaderEnd.blp1.lengths;
    }
    else {
        offs = header->HeaderEnd.blp2.offsets;
        lens = header->HeaderEnd.blp2.lengths;
    }

    
    blpjpeg::jpegCompress* pjpegCompress=NULL;
    if (savef == 0 || savef == 0x200) {
        //JPEG
        if (savef == 0)offs[0] = 0xa0;
        else offs[0] = 0x98;
        *(uint32*)dest = 0;
        dest += 4;
        pjpegCompress = (blpjpeg::jpegCompress*)(gData->buf + 4 * c);
        pjpegCompress->quality = gData->saveparam & 0xff;
    }
    else {
        if (savef == 0x100 || savef == 0x300|| savef == 0x500|| savef == 0x600) {
            if (savef == 0x100)offs[0] = 0x49c;
            else offs[0] = 0x494;
            memcpy(dest, gData->Palette, 1024);
            dest += 0x400;
        }
        else offs[0] = 0x94;
    }
    //convert
    uint32 height = gData->Height;
    uint32 width = gData->Width;
    uint32 length, a;
    uint32 currentI;
    uint32 maxI = gData->saveparam>>24;
    for (currentI = 0; ;) {
        uint32* sourced, *destd, s;

        //JPEG
        if (savef == 0 || savef == 0x200) {

            pjpegCompress->height = height;
            pjpegCompress->width = width;
            pjpegCompress->inbuff = gData->buf;
            pjpegCompress->workbuff = (short*)(pjpegCompress + 1);
            pjpegCompress->outbuff = dest;
            if ((height*width)<4096 || !(gData->saveparam&0xf000))
                blpjpeg::blpjpegCompressBase(pjpegCompress);
            else
                blpjpeg::blpjpegCompressProgressive(pjpegCompress);
            //length
            length = pjpegCompress->outlenght;
            dest += length;
        }
        //paletted
        else if ((savef == 0x100 || savef == 0x300|| savef == 0x500|| savef == 0x600)) {

            sourced = (uint32*)gData->buf;
            length = height*width;
            uint8* desta = dest + length;
            uint8 bof = 1;
            uint32 min, minj;
            long var;
            uint8 b;
            
            if(alphaDepth==1)*desta = 0;
            for (uint32 i = length; i--; sourced++, dest++) {
                a = *sourced;

                source = gData->Palette;
                min = 0xffff;
                for (uint32 j = 0; j<256; j++) {
                    b = (uint8)a;
                    var = b - *source++;
                    s = var * var;
                    b = (uint8)(a >> 8);
                    var = b - *source++;
                    s += var * var;
                    b = (uint8)(a >> 16);
                    var = b - *source++;
                    s += var * var;
                    source++;
                    if (s<min) { min = s; minj = j; }
                }

                *dest = (uint8)minj;
                //alpha
                if (alphaDepth) {
                    if (alphaDepth == 8)*desta++ = (uint8)(a >> 24);
                    else if (alphaDepth == 4) {
                        if (bof) {
                            uint8 b = (uint8)(a >> 28);
                            b &= 0xf;
                            *desta = b;
                        }
                        else {
                            uint8 b = (uint8)(a >> 24);
                            b &= 0xf0;
                            *desta |= b;
                            desta++;
                        }
                        bof = !bof;
                    }
                    else if (alphaDepth == 1) {
                        if ((long)a<0)*desta |= bof;
                        bof += bof;
                        if (!bof) {
                            bof++; desta++;*desta = 0;
                        }
                    }
                }
            }
            sourced = (uint32*)gData->buf;

            if (alphaDepth == 1)a = (length + 7) >> 3;
            else if (alphaDepth == 4)a = (length + 1) >> 1;
            else if (alphaDepth == 8)a = length;
            else a = 0;
            dest += a;
            //length
            length += a;
        }
        //dxt
        else {
            dxt::CompressDxt((DWORD*)gData->buf, dest, height, width, alphaDepth);
            //length
            if (alphaDepth)length = ((height + 3) & 0xfffffffc)*((width + 3) & 0xfffffffc);
            else length = ((height + 3) >> 2)*((width + 3) >> 2) * 8;
            dest += length;
        }


        lens[currentI] = length;
        if ((width + height)>2) {
            length += offs[currentI];
            offs[++currentI] = length;
            if (currentI == maxI) {
                currentI--;
                break;
            }
        }
        else break;
        //resize
        sourced = destd = (uint32*)gData->buf;
        if (width == 1) {
            if (height != 1) {
                height = height >> 1;
                for (uint32 y = 0; y<height; y++) {
                    s = *sourced++;
                    a = (s & 0xff00ff00) >> 8;
                    uint32 a1 = s & 0xff00ff;
                    s = *sourced++;
                    a += (s & 0xff00ff00) >> 8;
                    a1 += s & 0xff00ff;
                    destd[y] = ((a << 7) & 0xff00ff00) + ((a1 >> 1) & 0xff00ff);
                }
            }
        }
        else if (height == 1) {
            if (width != 1) {
                width = width >> 1;
                for (uint32 x = 0; x<width; x++) {
                    s = *sourced++;
                    a = (s & 0xff00ff00) >> 8;
                    uint32 a1 = s & 0xff00ff;
                    s = *sourced++;
                    a += (s & 0xff00ff00) >> 8;
                    a1 += s & 0xff00ff;
                    destd[x] = ((a << 7) & 0xff00ff00) + ((a1 >> 1) & 0xff00ff);
                }
            }
        }
        else {
            height = height >> 1;
            for (uint32 y = 0; y<height; y++) {
                for (uint32 x = 0; x<(width >> 1); x++) {
                    s = *sourced;
                    a = (s & 0xff00ff00) >> 8;
                    uint32 a1 = s & 0xff00ff;
                    s = *(sourced + width);
                    a += (s & 0xff00ff00) >> 8;
                    a1 += s & 0xff00ff;
                    sourced++;
                    s = *sourced;
                    a += (s & 0xff00ff00) >> 8;
                    a1 += s & 0xff00ff;
                    s = *(sourced + width);
                    a += ((s & 0xff00ff00) >> 8) + 1;
                    a1 += (s & 0xff00ff) + 1;
                    *destd++ = ((a << 6) & 0xff00ff00) + ((a1 >> 2) & 0xff00ff);
                    sourced++;
                }
                if (width & 1)sourced++;
                sourced += width;
            }
            width = width >> 1;
        }
    }
    //compress the jpg headers
    if ((savef == 0 || savef == 0x200) && currentI>1) {
        dest = (BYTE*)header;
        DWORD bn;
        for (bn = 0; bn < lens[currentI]; bn++) {
            BYTE cb = *(dest + offs[0] + bn);
            DWORD i;
            for (i = currentI; i; i--) {
                if (cb != *(dest + offs[i] + bn))break;
            }
            if (i)break;
        }
        if (savef == 0)*(DWORD*)(dest + 0x9c) = bn;
        else *(DWORD*)(dest + 0x94) = bn;
        lens[0] -= bn;
        offs[0] += bn;
        for (DWORD i = 1; i <= currentI; i++) {
            lens[i] -= bn;
            memmove(dest + offs[i] - bn*(i - 1), dest + offs[i] + bn, lens[i]);
            offs[i] -= bn*(i - 1);
        }
    }


    a = offs[currentI] + lens[currentI];
    for (; (height > 1) || (width > 1);) {
        height >>= 1; width >>= 1;
        ++currentI;
        offs[currentI] = offs[currentI - 1];
        lens[currentI] = lens[currentI - 1];
    }

    for (; ++currentI < 16;)offs[currentI] = lens[currentI] = 0;

    WriteFl(header,a,0);
}
static void DoWriteFinish(void) {

}





